#include "hw_iface.h"
#include "hw_impl_zybo.h"
#include "type_def.h"

#define LIDARLITE_DEVICE_ADDR		0x62
#define LIDAR_OFFSET 0.02 // Distance from LiDAR sensor to ground, in meters

int lidarlite_write(struct I2CDriver *i2c, u8 register_addr, u8 data);
int lidarlite_read(struct I2CDriver *i2c, u8* recv_buffer, u8 register_addr, int size);

int zybo_lidar_reset(struct LidarDriver *self) {
  struct I2CDriver *i2c = self->i2c;

  int status = 0;

  // Device Reset & Wake up with default settings
  status = lidarlite_write(i2c, 0x00, 0x00);
  usleep(15000);

  // Enable Free Running Mode and distance measurements with correction
  status |= lidarlite_write(i2c, 0x11, 0xff);
  status |= lidarlite_write(i2c, 0x00, 0x04);

  return status;
}

int zybo_lidar_read(struct LidarDriver *self, lidar_t *lidar) {
  struct I2CDriver *i2c = self->i2c;
  u8 buf[2];
  int status = 0;

  // Read the sensor value
  status = lidarlite_read(i2c, buf, 0x8f, 2);
  float distance_cm = (float)(buf[0] << 8 | buf[1]) - LIDAR_OFFSET;
  lidar->distance_m = distance_cm * 0.01;

  return status;
}

////////////////////
// Helper functions
////////////////////

int lidarlite_write(struct I2CDriver *i2c, u8 register_addr, u8 data) {
  u8 buf[] = {register_addr, data};
  return i2c->write(i2c, LIDARLITE_DEVICE_ADDR, buf, 2);
}

int lidarlite_read(struct I2CDriver *i2c, u8* recv_buffer, u8 register_addr, int size) {
  u8 buf[] = {register_addr};
  int status = 0;

  status |= i2c->write(i2c, LIDARLITE_DEVICE_ADDR, buf, 1);
  status |= i2c->read(i2c, LIDARLITE_DEVICE_ADDR, recv_buffer, size);
  return status;
}

// Maybe this will be useful?
int lidarlite_sleep(struct I2CDriver *i2c) {
  return lidarlite_write(i2c, 0x65, 0x84);
}
